#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
GUI for AOV Period Search in Light Curves

Author: Alex Schwarzenberg-Czerny, alex@camk.edu.pl

To be done:  
   -add flattening of transit light curves by ort fit with
    clipping of outstanding points.
   -use showFullScreen() on your widget
"""

# Packages
import sys, os, random
import numpy as np
from scipy import *
from pprint import pprint
#import atpy
#import asciitable as asc
import platform
#>>> platform.system()
from PyQt6.QtCore import *
from PyQt6.QtWidgets import *
from PyQt6.QtGui import *

import matplotlib
from pylab import plot, show, ylim, yticks, getp
matplotlib.use('QtAgg')
#from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg
from matplotlib.backends.backend_qtagg import \
    FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qtagg import \
    NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
from PIL import Image

#import aov as _aov
import pyaov

class AppForm(QMainWindow):

    def __init__(self, parent=None): # Initialize program
        QMainWindow.__init__(self, parent)
#        screen = QDesktopWidget().screenGeometry()
#        size = self.geometry()
#        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2) 
#        self.setGeometry(int(screen.width()*0.7), int(screen.height()*0.7), \
#               screen.width()/2, screen.height()/2)
        self.strold = ""
        self.click = 'None'
        self.marg = 0.07
        self.setWindowTitle('AOV Period Search')
        self.create_menu()
        self.create_main_frame()
        self.create_status_bar()

        #        self.connect(self.button_open, SIGNAL('clicked()'), \
        #           self.file_dialog)
        self.open_dialog() # open file & set default values

    def open_dialog(self): # Action for File>Open
        fd = QFileDialog(self)
        self.plik = str(fd.getOpenFileName()[0])
        fd.close()
        self.input_lc(self.plik) # input light curve
        self.yd0 = pyaov.normalize(self.yd,self.ed,var=0.) 

        # initialize parameters
        self.fup0,self.fstep0,self.flow0 = pyaov.fgrid(self.xd)
        self.fup0 = 1.5*self.fup0 
        self.fup,self.fstep,self.flow = \
            self.fup0,self.fstep0,self.flow0
        self.phlow0, self.phup0 = (0., 1.2)
        self.phlow, self.phup = (self.phlow0, self.phup0)
        self.clicks = 0
        self.nh2 = 3
        self.nh2box.setText("%3d" % self.nh2)

        # get periodogram
        self.on_draw_per()

    def freeze_dialog(self): # Action for File>Freeze & Open
        fd = QFileDialog(self)
        self.plik = str(fd.getOpenFileName())
        fd.close()
        self.input_lc(self.plik) # input light curve
        self.yd0 = pyaov.normalize(self.yd,self.ed,var=0.)  

        # freeze & get periodogram
        self.fup0,self.fstep0,self.flow0 = \
            self.fup,self.fstep,self.flow
        self.phup0,self.phlow0 = self.phup,self.phlow
        self.clicks = 0
        self.on_draw_per()

    def save_plot(self): # Action for File>Save Plot
        file_choices = "png (*.PNG)|*.png"
        
        path = QFileDialog.getSaveFileName(self, 'Save file','', file_choices)
        path = path[0]+'.png'
        if path:
#            self.canvas.print_figure(path, dpi=self.dpi)
             self.canvas.draw()
             rgba = np.asarray(self.canvas.buffer_rgba())
        # ... and pass it to PIL.
             im = Image.fromarray(rgba)
        # This image can then be saved to any format supported by Pillow, e.g.:
             im.save(path)            
             self.statusBar().showMessage('Saved to %s' % path)
    
    def save_data(self): # Action for File>Save Data
        file_choices = "DAT (*.dat)|*.dat"
        
        path = QFileDialog.getSaveFileName(self,'Save data', '', file_choices)
        if path:
#            asc.write({'x': self.xd, 'y': self.yd, 'z': self.ed}, \
#                 path,Writer=asc.NoHeader)
            np.savetxt(path,np.transpose([self.xd, self.yd, \
                 self.ed]))
            self.statusBar().showMessage('Saved to %s' % path, 2000)
    
    def save_period(self): # Action for File>Save Periodogram
        file_choices = "DAT (*.per)|*.per"
        
        path = QFileDialog.getSaveFileName(self, 'Save periodogram', '', \
            file_choices)
        if path:
#            asc.write({'x': self.fr, 'y': self.per}, \
#                 path,Writer=asc.NoHeader)
            np.savetxt(path,np.transpose([self.fr, self.per]))
            self.statusBar().showMessage('Saved to %s' % path, 2000)
    
    def on_about(self): # Action on About>Help
        msg = """ AOV Period Search of Light Curve
   To start this program e.g. type: python aovgui.py | tee aov.log. In this way messages would be written both to the screen and to the aov.log file. You may 
comment your log with NOTEs in the bottom box.
   * Use File>OPEN to input data in 2 or 3 ascii columns (time,value[,error]);
# commented headers are permitted.
   * Top and bottom plots present respectively periodogram and data  folded with
the peak frequency (marked with dot)
   * Try pointing other peaks with mouse (within 0.1-0.9 y-scale) or type 
frequency to change data folding
   * Zoom-in/out the plots by clicking bottom/top frame on two locations to select the range. Zoom-in would cover the selected range while zoom-out would expand the x-scale asymmetricaly by the ratio of original and selected ranges...
   * ...or refine your frequency by fitting trig polynomial series. To obtain 
a realistic estimate of the frequency error, use REFINE when displaying moderate 
range of amh|aov|atr periodogram, centered on the line: median of the displayed  spectrum is used to estimate noise level.
   * individual observations may be rejected/re-included by clicking them on the 
LC plot. Note, this plot can also be zoomed. Enter 0. frequency to plot LC against time [time unit=data span]
   * PREWHITEN the data with the specified frequency
   * Fit data with FOURIER series, while frequency remains fixed. For frequency adjustment use refine, Fourier fit is prone to correlation of coefficients and frequency errors. 
   * RESET the plot scales to initial or last freeze by clicking button
   * Select periodogram method by ticking AMH|AOV|ATR|POW|WIN marks
   * Drag the SLIDER or type NH2 to modify no. of bins|harmonics*2
   * SAVE the (prewhitened?) DATA to a file using the File>Save Data menu
   * SAVE the PLOT to a file using the File>Save Plot menu
   * Use File>FREEZE & OPEN to freeze periodogram parameters and input new data 
   * Click on a bar to receive an informative message
        """
        QMessageBox.about(self, "About AOV Search", msg.strip())
    
    def on_author(self): # Action on About>Help
        msg = """ AOV Period Search of Light Curve
        
        Author: Alex Schwarzenberg-Czerny, alex@camk.edu.pl
        Reference:
        Alex Schwarzenberg-Czerny, 1998, Baltic Astronomy, v.7, p.43-69.

        Licence: This python code is in the public domain. The associated Fortran routines are copyrighted (C) by Alex Schwarzenberg Czerny and put into your free use, provided they remain unchanged, and are not used for purposes carrying risk of any damages.
        History: 
        Origin: 19.10.2012
        Modified: 26.10.2012 - general streamline of windows
        Modified: 12.11.2012 - frequency error estimate in REFINE and NOTE box
        Modified: 01.07.2023 - thorough revision and upgrade to Python3
        Modified: 16.12.2023 - upgrade of graphics to PyQt6
        

        Acknowledgements:
        This routine embeds a matplotlib plot into PyQt5 GUI application following a demo by Eli Bendersky.
        Pyaov.py wrapper of f95 periodogram follows an example by Ewald Zietsman
        """
        QMessageBox.about(self, "AOV Search Package: History", \
           msg.strip())
    
    def on_click(self, event):
        # The event received here is of the type
        # matplotlib.backend_bases.PickEvent
        #
        # It carries lots of information, of which we're using
        # only a small amount here.
        # 
# =============Periodogram plot editing=============================
        if (event.inaxes == self.ax_per and self.click!='lc'):
           self.click='per'
           self.clicks = self.clicks+1
           xlow,xup =  getp(self.ax_per,'xbound')
           xspan = (xup-xlow)
           xmarg = xspan*self.marg
           ylow,yup =  getp(self.ax_per,'ybound')
           yspan = (yup-ylow)
           ymarg = yspan*self.marg
           if ((xup-event.xdata)<xmarg):
              event.xdata = xup
           if ((event.xdata-xlow)<xmarg):
              event.xdata = xlow
           if ((yup-event.ydata)<ymarg):
              event.ydata = yup
           if ((event.ydata-ylow)<ymarg):
              event.ydata = ylow
           if (self.clicks == 1):
               self.xsav=event.xdata
               self.ysav=event.ydata
# fold lc with marked period
               if (yup!=event.ydata and event.ydata!=ylow):
                  self.clicks = 0 
                  n=len(self.fr)
                  ib=int(n*0.035)
                  ia=max(int((event.xdata-self.flow)/self.fstep) \
                     -ib,0)
                  ib=min(ia+ib+ib,n-1)
                  self.frmax,fm,dx = pyaov.peak(self.per[ia:ib])
                  self.frmax = self.frmax*self.fstep+self.fr[ia]
                  self.ax_per.plot(self.frmax, 0.5*fm, 'r-o') 
                  self.textbox.setText("%18.10g" % self.frmax)
                  self.click='None'
                  self.on_draw_lc()
               else:
                  self.ax_per.plot(event.xdata, event.ydata, 'r-o') 
# resize periodogram
           if self.clicks == 2: 
               self.clicks = 0
               if(abs(self.xsav-event.xdata)<xmarg):
                  self.xsav=(self.xsav+event.xdata)*0.5
                  event.xdata=min(max(self.xsav-xmarg*0.5,xlow), \
                     xup-xmarg)
                  self.xsav=event.xdata+xmarg
# zoom in periodogram between marks or...
               self.fup=max(self.xsav,event.xdata)
               self.flow=min(self.xsav,event.xdata)
               if (self.ysav==yup):
# ...zoom out periodogram by the span ratio
                  ratio=xspan/abs(event.xdata-self.xsav)
                  self.fup=self.fup+ratio*(xup-self.fup)
                  self.flow=max(self.flow-ratio*(self.flow-xlow),0)
               self.ax_per.plot(event.xdata, event.ydata, 'r-o') 
               self.fstep=min((self.fup-self.flow)/300.,self.fstep0)
               self.click='None'
               self.on_draw_per()           
           self.canvas.draw()
# =============Light curve plot editing=============================
        elif (event.inaxes == self.ax_lc and self.click!='per'):
           self.click='lc'
           self.clicks = self.clicks+1
           xlow,xup =  getp(self.ax_lc,'xbound')
           xspan = (xup-xlow)
           xmarg = xspan*self.marg
           ylow,yup =  getp(self.ax_lc,'ybound')
           yspan = (yup-ylow)
           ymarg = yspan*self.marg
           if ((xup-event.xdata)<xmarg):
              event.xdata = xup
           if ((event.xdata-xlow)<xmarg):
              event.xdata = xlow
           if ((yup-event.ydata)<ymarg):
              event.ydata = yup
           if ((event.ydata-ylow)<ymarg):
              event.ydata = ylow
           if (self.clicks == 1):
               self.xsav=event.xdata
               self.ysav=event.ydata
# toggle point validity or ...
               if (yup!=event.ydata and event.ydata!=ylow):
# find nearby point
                  self.clicks = 0 
                  b = np.where(abs(self.ph-event.xdata)<2*xmarg)[0]
                  r = np.sqrt(np.square((self.ph[b]- event.xdata)/ \
                       xmarg)+np.square((self.phy[b]-event.ydata)/ \
                       ymarg))
                  ind = self.idx[b[r.argmin()]]%len(self.xd)
                  self.ed[ind]=-self.ed[ind]
                  self.click='None'
                  self.on_draw_lc()
               else:
                  self.ax_lc.plot(event.xdata, event.ydata, 'r-o') 
# ...resize light curve plot
           if self.clicks == 2: 
               self.clicks = 0
               if(abs(self.xsav-event.xdata)<xmarg):
                  self.xsav=(self.xsav+event.xdata)*0.5
                  event.xdata=min(max(self.xsav-xmarg*0.5,xlow), \
                     xup-xmarg)
                  self.xsav=event.xdata+xmarg
# zoom in light curve between marks or...
               self.phup=max(self.xsav,event.xdata)
               self.phlow=min(self.xsav,event.xdata)
               if (self.ysav==yup):
# ...zoom out light curve by the span ratio
                  ratio=xspan/abs(event.xdata-self.xsav)
                  self.phup=self.phup+ratio*(xup-self.phup)
                  self.phlow=max(self.phlow-ratio*(self.phlow- \
                     xlow),0)
               self.ax_lc.plot(event.xdata, event.ydata, 'r-o') 
               self.click='None'
               self.on_draw_lc()           
           self.canvas.draw()
        else:
           print ("Click ignored, first finish action in " +self.click+ " plot")

    def on_reset(self): # Reset intervalper & lc grids
        self.fup,self.fstep,self.flow = \
           self.fup0,self.fstep0,self.flow0
        self.phup,self.phlow = self.phup0,self.phlow0
        self.on_draw_per()

    def on_refine(self): # Refine frequency by orthogonal polynomials fit
        # initial data from box text
        print ("Refine\n")
        nu = float(self.textbox.text())  
        if nu == 0.: 
           print('on_refine: error-zero nu')
        backgnd = np.median(self.per)
        nh2new = 1
        ndx = self.ed > 0.
        n = len(ndx)
        valout=self.yd0[ndx]/self.ed[ndx]
        snew = np.sum(np.square(valout))
        fnew = 1000.
        next = True

        while next: # fit a longer orthogonal polynomial series 
           nh2old=nh2new
           sold=snew
           fold=fnew

           nh2new=int(max(float(nh2old-1)*0.7, \
              float((nh2old-1)/2)+1.1))*2+1
           valin=self.yd0
           fr, dfr, valout=pyaov.prew(self.xd, valin, self.ed, nu, \
               bacgnd=backgnd, nh2=nh2new)
           snew = np.sum(np.square(valout[ndx]/self.ed[ndx]))
           fnew = (sold-snew)*(n-nh2new)/((nh2new-nh2old)*snew)

           if ( (fnew/fold < 1.) or (nh2new < 4) ): # still improving fit
               next = True
               nu = fr
               self.textbox.setText("%18.10g" % nu)
           else:
               next = False
        print ('refined with nh2= ',nh2new)
        self.on_draw_lc()
    
    def on_prewhite(self): 
        # prewhite data with fixed frequency by subtraction of
        # trig orthogonal polynomial series

        # frequency from box text
        print ('Prewhite\n')
        nu = float(self.textbox.text())  
        if nu == 0.: 
           print('on_prewhite: error-zero nu')

        fr,dfr,self.yd=pyaov.prew(self.xd, self.yd0, \
               self.ed,-nu,bacgnd=0.,nh2=self.nh2) # nu<0 not adjusted
        self.yd0 = pyaov.normalize(self.yd,self.ed,var=0.)  

        # get periodogram
        self.on_draw_per()
    
    def on_fourier(self): 
        # prewhite data with fixed frequency by subtraction of
        # Fourier series

        # frequency from box text
        print ('Fourier\n')
        nu = float(self.textbox.text())  
        if nu == 0.: 
           print('on_fourier: error-zero nu\n')

        backgnd = np.median(self.per)
        fr,dfr,yd,cof,dcof=pyaov.fouw(self.xd, self.yd0, \
               self.ed,-nu,bacgnd=backgnd,nh2=self.nh2) # nu<0 not adjusted 
    
    def on_draw_lc(self):
        """ Redraws the folded light curve
        """
        # data from box text
        nu = float(self.textbox.text())
        if nu == 0.: # print raw light curve
           nu = 1./(self.xd.max()-self.xd.min())

        # clear the axes and redraw the plot anew
        self.ax_lc.clear()        
        self.ax_lc.grid(self.grid_cb.isChecked())

        imin=self.yd.argmin()
        x = (self.xd-self.xd[imin])*nu+0.5
        x = x-np.floor(x)
        y = self.yd
        e = self.ed
        if (self.phup>1.):
            x = np.append(x,np.add(x,1.))
            y = np.append(y,y) 
            e = np.append(e,e)
        self.idx = np.where(np.logical_and \
            (x>self.phlow,x<self.phup))[0]
        self.ph  = x[self.idx]      
        self.phy = y[self.idx]
        self.phe = e[self.idx]

        b = self.phe>0.
        self.ax_lc.plot(self.ph[b], self.phy[b], 'g o') 
        b = self.phe<=0.
        self.ax_lc.plot(self.ph[b], self.phy[b], 'r o') 
        self.ax_lc.set_xlim(self.phlow, self.phup)
        yup=self.phy.max()
        ylow=self.phy.min()
        yspan=(yup-ylow)*(0.5+3*self.marg)
        yav=(yup+ylow)*0.5
        self.ax_lc.set_ylim(yav-yspan, yav+yspan)
        self.canvas.draw()

    def on_slide_nh2(self):
        nh2 = max(int(10.**(self.slider.value()/100.*1.6)+0.5)+3,3)
        self.nh2box.setText("%3d" % nh2)
        self.canvas.draw()
    
    def on_edit_nh2(self):
        nh2 = max(int(self.nh2box.text()),3)
        if (nh2 != self.nh2):
           self.nh2 = nh2
           self.nh2box.setText("%3d" % nh2)
           self.on_draw_per() 
    
    def on_draw_per(self):
        """ Redraws the periodogramme
        """
        meth = self.radioGroup.checkedId()
        self.nh2 = max(int(self.nh2box.text()),3)
        self.nh2box.setText("%3d" % self.nh2)
        print ('method= ','amh aov atr pow win'.split()[meth],'\n')

        self.ax_per.clear()        
        self.ax_per.grid(self.grid_cb.isChecked())
        # Calculate perioddogram
        if meth == 0:
           self.per,self.fr,self.frmax = \
              pyaov.amhw(self.xd,self.yd0,self.ed,  \
              self.fup,self.fstep,fr0=self.flow,nh2=self.nh2) 
        elif meth == 1:
           self.per,self.fr,self.frmax = \
              pyaov.aovw(self.xd,self.yd0,self.ed,  \
              self.fup,self.fstep,fr0=self.flow,nh2=self.nh2) 
        elif meth == 2:
           self.per,self.fr,self.frmax = \
              pyaov.atrw(self.xd,self.yd0,self.ed,  \
              self.fup,self.fstep,fr0=self.flow,nh2=self.nh2) 
        elif meth == 3:
           self.per,self.fr,self.frmax = \
              pyaov.pspw(self.xd,self.yd0,self.ed,  \
              self.fup,self.fstep,fr0=self.flow) 
        elif meth == 4:
           self.per,self.fr,self.frmax = \
              pyaov.pspw(self.xd,np.ones(self.xd.size,dtype=float), \
              self.ed,self.fup,self.fstep,fr0=self.flow) 
        else:
           print ('on_draw_per: Wrong mode')

        # set new frequency and recalculate lc
        self.ax_per.plot(self.fr, self.per, \
           linestyle='solid')        
        self.ax_per.plot(self.frmax, 0.5*np.amax(self.per), 'r-o')
        self.canvas.draw()
        self.textbox.setText("%18.10g" % self.frmax)
        self.on_draw_lc()

    def on_note(self):
        stri = self.notebox.text()
        if (stri != self.strold and stri != " "):
          print (stri)
          self.notebox.setText(" ")
          self.canvas.draw()
          self.strold = stri
    
    def create_main_frame(self): # Organize the main window
        self.main_frame = QWidget()
        
        # Create the mpl Figure and FigCanvas objects. 
        # 5x4 inches, 100 dots-per-inch
        #
        self.dpi = 100
#        screen = QtGuiQDesktopWidget().screenGeometry()
        ag=QGuiApplication.primaryScreen().availableGeometry()
        self.fig = Figure((ag.width()*0.7/self.dpi, \
            ag.height()*0.7/self.dpi), dpi=self.dpi)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        
        # Since we have only one plot, we can use add_axes 
        # instead of add_subplot, but then the subplot
        # configuration tool in the navigation toolbar wouldn't
        # work.
        #
        self.ax_per = self.fig.add_subplot(211)
        self.ax_lc = self.fig.add_subplot(212)
        
        # Bind the 'pick' event for clicking on one of the bars
        cid = self.fig.canvas.mpl_connect('button_press_event', \
                                  self.on_click)
        
        # Create the navigation toolbar, tied to the canvas
        #
        self.mpl_toolbar = NavigationToolbar(self.canvas, \
            self.main_frame)
        
# Row of GUI view controls ===========================
        
        # edited frequency field
        freq_label = QLabel("Freq =")
        self.textbox = QLineEdit()
        self.textbox.setMinimumWidth(200)
#        self.connect(self.textbox, SIGNAL('editingFinished ()'), \
#                 self.on_draw_lc)
        self.textbox.editingFinished.connect(self.on_draw_lc)
        
        # refine frequency button
        self.refine_button = QPushButton("&Refine")
        self.refine_button.clicked.connect(self.on_refine)
        
        # prewhite button
        self.prewhite_button = QPushButton("&Prewhite")
#        self.connect(self.prewhite_button, SIGNAL('clicked()'), \
#            self.on_prewhite)
        self.prewhite_button.clicked.connect(self.on_prewhite)
        
        # Fourier coefficients button
        self.fourier_button = QPushButton("&Fourier")
#        self.connect(self.fourier_button, SIGNAL('clicked()'), \
#            self.on_fourier)
        self.fourier_button.clicked.connect(self.on_fourier)

        # zoom out button
        self.zoom_button = QPushButton("&Reset")
#        self.connect(self.zoom_button, SIGNAL('clicked()'), \
#            self.on_reset)
        self.zoom_button.clicked.connect(self.on_reset)

        # show grid button
        self.grid_cb = QCheckBox("Show &Grid")
        self.grid_cb.setChecked(False)
#        self.connect(self.grid_cb, SIGNAL('stateChanged(int)'), \
#            self.on_draw_per)
        self.grid_cb.stateChanged.connect(self.on_draw_per)
                
        # Layout of GUI view controls bar 
        hbox = QHBoxLayout()
        
        for w in [  freq_label, self.textbox, self.refine_button, \
                self.prewhite_button, self.fourier_button, \
                self.zoom_button, self.grid_cb]:
            hbox.addWidget(w)
            hbox.setAlignment(w, Qt.AlignmentFlag.AlignVCenter)
        
        vbox = QVBoxLayout()
        vbox.addWidget(self.canvas)
        vbox.addWidget(self.mpl_toolbar)
        vbox.addLayout(hbox)

# Row of method controls as RadioButtons =========================
        listOfChoices = 'amh aov atr pow win'.split()
        self.radioGroup = QButtonGroup()
        self.radioGroup.setExclusive(True)
        rbox = QHBoxLayout()

        bIsFirst = True
        nMaxLen = 0
        for i,row in enumerate(listOfChoices):
           radio = QRadioButton(row)
           self.radioGroup.addButton(radio, i)
           if bIsFirst:
               radio.setChecked(True)
               bIsFirst = False
           if len(row) > nMaxLen:
               nMaxLen = len(row)
#           self.connect(radio, SIGNAL("toggled(bool)"), \
#                 self.on_draw_per)
           radio.toggled.connect(self.on_draw_per)
           rbox.addWidget(radio)
           rbox.setAlignment(radio, Qt.AlignmentFlag.AlignVCenter)

        # slider field
        self.slider = QSlider(Qt.Orientation.Horizontal)
        self.slider.setRange(0, 100)
        self.slider.setValue(0)
        self.slider.setTracking(True) # may call while mooving
        self.slider.setTickPosition(QSlider.TickPosition.TicksBothSides)
#        self.connect(self.slider, SIGNAL('valueChanged(int)'), \
#             self.on_slide_nh2)
        self.slider.valueChanged.connect(self.on_slide_nh2)
        rbox.addWidget(self.slider)
        rbox.setAlignment(self.slider, Qt.AlignmentFlag.AlignVCenter)

        # edited nh2 field
        nh2_label = QLabel("nh2=")
        rbox.addWidget(nh2_label)
        rbox.setAlignment(nh2_label, Qt.AlignmentFlag.AlignVCenter)
        self.nh2box = QLineEdit()
        self.nh2box.setMinimumWidth(30)
#        self.connect(self.nh2box, SIGNAL('editingFinished ()'), \
#                 self.on_edit_nh2)
        self.nh2box.editingFinished.connect(self.on_edit_nh2)
        rbox.addWidget(self.nh2box)
        rbox.setAlignment(self.nh2box, Qt.AlignmentFlag.AlignVCenter)

        vbox.addLayout(rbox)

# Row for notes  ============================================            
        note_label = QLabel("Note:")
        self.notebox = QLineEdit()
        self.notebox.setMinimumWidth(int(ag.width()*0.8))
#        self.connect(self.notebox, SIGNAL('editingFinished ()'), \
#                 self.on_note)
        self.notebox.editingFinished.connect(self.on_note)

        nbox = QHBoxLayout()
        nbox.addWidget(note_label)
        nbox.setAlignment(note_label, Qt.AlignmentFlag.AlignVCenter)
        nbox.addWidget(self.notebox)
        nbox.setAlignment(self.notebox, Qt.AlignmentFlag.AlignVCenter)

        vbox.addLayout(nbox)

# End menu rows ==========================================

        self.main_frame.setLayout(vbox)
        self.setCentralWidget(self.main_frame)
    
    def create_status_bar(self):
        self.status_text = QLabel( \
            "by Alex Schwarzenberg-Czerny, 2012")
        self.statusBar().addWidget(self.status_text, 1)
        
    def create_menu(self):  # Create File> and Help> Menus      
        self.file_menu = self.menuBar().addMenu("&File")
        
        open_file_action = self.create_action("&Open file",
            shortcut="Ctrl+O", slot=self.open_dialog, 
            tip="Open ascii data file (time, value columns)")
        freeze_file_action = self.create_action("&Freeze and Open",
            shortcut="Ctrl+F", slot=self.freeze_dialog, 
            tip="Freeze parameters & open ascii data file")
        out_data_action = self.create_action("&Save data",
            shortcut="Ctrl+D", slot=self.save_data, 
            tip="Save the prewhitened(?) data")
        out_period_action = self.create_action("&Save periodogram",
            shortcut="Ctrl+P", slot=self.save_period, 
            tip="Save the current periodogram values")
        out_fig_action = self.create_action("&Save plot",
            shortcut="Ctrl+S", slot=self.save_plot, 
            tip="Save the plot")
        quit_action = self.create_action("&Quit", slot=self.close, 
            shortcut="Ctrl+Q", tip="Close the application")
        
        self.add_actions(self.file_menu, 
            (open_file_action, freeze_file_action, out_data_action, \
                out_period_action, out_fig_action, None, \
                quit_action))
        
        self.help_menu = self.menuBar().addMenu("&Help")
        about_action = self.create_action("&About", 
            shortcut='F1', slot=self.on_about, 
            tip='About the demo')
        author_action = self.create_action("A&uthor", 
            shortcut='Ctrl+A', slot=self.on_author, 
            tip='Author & History')
        self.add_actions(self.help_menu, \
            (about_action,author_action))

    def add_actions(self, target, actions): # Menus support
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    def create_action(  self, text, slot=None, shortcut=None, 
                        icon=None, tip=None, checkable=False):
        # more Menus support
        act_signal=pyqtSignal(list)
        action = QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(":/%s.png" % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
#            self.connect(action, SIGNAL(signal), slot)
            action.triggered.connect(slot)
        if checkable:
            action.setCheckable(True)
        return action
    
    def input_lc(self,filename): # input data from file
        # read data file
        z = np.loadtxt(filename,dtype=np.float64)
        n = len(z)
        if (n > 0):
           if (len(z[0,:]) >= 2):
              self.xd = z[:,0]
              self.yd = z[:,1]
              if (len(z[0,:]) >= 3):
                 self.ed = z[:,2]
                 n = n - len(np.where(self.ed <= 0.)[0])
                 print ('Assumed: time,value & error columns' \
                    ' (errors-->weights)')
              else:
                 self.ed = np.ones(n,dtype=float)
                 print ('Assumed: time,value columns (no weights)')
           else:
              print ('Wrong data:less than 2 columns')
        if (n < 4):
           print ('Wrong data:too few observations')

def main(): # Main execution loop
    app = QApplication(sys.argv)
    form = AppForm()
    form.show()
    app.exec()

if __name__ == "__main__":
    main()

